package SiteUtils;
require Exporter;
use HTML::Entities;
use Captcha::reCAPTCHA;

# Constants used in this file and in PubNet Perl scripts.

use constant DOMAIN => "pubnet.gersteinlab.org";
use constant TYNA => "http://yeasthub2.gersteinlab.org/tyna/index.jsp?";
use constant PUBLIC_KEY => "6LfFRwIAAAAAAKCPIA5VFr-7LaXsR8QrI45EuV6N";
use constant PRIVATE_KEY => "6LfFRwIAAAAAAF4mQIufWAfKk9Fold-vIdNVIDBD";
use constant P_OUT => "../data/out";  
use constant H_OUT => "../out";
use constant P_RECENT => "../data/recent"; 
use constant H_RECENT => "../recent";
use constant P_SAVED => "../data/saved";
use constant H_SAVED => "../saved";
use constant LAST16 => "../data/last16";

our @ISA = qw(Exporter);
our @EXPORT = qw(PUBLIC_KEY PRIVATE_KEY DOMAIN P_OUT H_OUT P_RECENT H_RECENT P_SAVED H_SAVED LAST16
   pageHead pageFoot captchaCheck URLEncode viewQuery viewDetails footLinks getSource);


sub pageHead($) {
    my $title = $_[0];
    print "Content-type: text/html\n\n";
    print <<EOF;
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en-us" lang="en-us">
<head>
<title>$title</title>
<link rel="stylesheet" type="text/css" href="../site.css" />
<script type="text/javascript" src="../site.js"></script>
</head>
<body>
EOF
}

sub pageFoot() {
    my $domain = DOMAIN;
    print <<EOF;
<div class="nav"><a class="nav" href="http://$domain/">PubNet Home</a></div>
</body>
</html>
EOF
}

sub captchaCheck($) {
    my %p = %{$_[0]};  # CGI parameters to be passed again as hidden inputs.
    my $c = Captcha::reCAPTCHA->new;
    my $error = undef;
    # Verify submission
    if ( $p{ 'recaptcha_response_field' } ) {
	my $result = $c->check_answer(PRIVATE_KEY, $ENV{'REMOTE_ADDR'}, $p{ 'recaptcha_challenge_field' }, $p{ 'recaptcha_response_field' } );

	if ( $result->{is_valid} ) {
	    return 1;
	}
	else {
        # Error
	    $error = $result->{error};
	}

    }
# Output form
    pageHead("PubNet - Security Step");
    print <<EOF;

<h1>Please Help Us Keep PubNet Free of Spam</h1>
<form action="" method="post">
EOF

    print $c->get_html(PUBLIC_KEY,  $error );

    print <<EOF;
<p><input type="submit" value="submit" /></p>
<div>
EOF
foreach $key (keys(%p)) {
    print qq[ <input type="hidden" name="$key" value="$p{$key}" />\n ];
}
    print <<EOF;
</div>
<p><a class="foot" href="http://pubnet.gersteinlab.org">PubNet Home</a></p>
</form>
</body>
</html>
EOF
    return 0;
}


sub URLEncode {
   my $theURL = $_[0];
   $theURL =~ s/(\W)/"%" . uc(sprintf("%2.2x",ord($1)))/eg;
   return $theURL;
}



sub viewQuery( $ $ $ $ $ $ ) {

    my ($id, $h_id, $query1, $query2, $node, $edge) = @_;

    my $query1enc = encode_entities( $query1, '<>&"' );
    my $query2enc = encode_entities( $query2, '<>&"' );
    my $query1url = URLEncode( $query1 );
    my $query2url = URLEncode( $query2 );
    my $alt = ($query2) ? "$query1enc $query2enc" : $query1enc;

    print <<EOF;

<table style="margin-left: auto; margin-right: auto;">

<tr>
<td colspan="6" style="color: #009900; background-color: #80ff80; padding: 20px;"><img 
    style="border-width: 0pt;" src="$h_id\_small.png" title="$id" alt="$alt" 
/></td>
</tr>

<tr>
<td class="svg"><a href="$h_id.svg" class="svg" onclick="return popup(this.href);">SVG</a></td>
<td class="png"><a href="$h_id.png" class="png" onclick="return popup(this.href);">PNG</a></td>
<td class="ps"><a href="$h_id.ps" class="ps" onclick="return popup(this.href);">PS</a></td>
<td class="pdf"><a href="$h_id.pdf" class="pdf" onclick="return popup(this.href);">PDF</a></td>
<td class="gdl"><a href="$h_id.gdl" class="gdl" onclick="return popup(this.href);">GDL</a><br /></td>
<td class="zip"><a href="$h_id.zip" class="zip">ZIP</a><br /></td>
</tr>

<tr>
<th colspan="6" class="gallery-title">S E L E C T &nbsp; &nbsp; F O R M A T</th>
</tr>
</table>

<div id="gallery-details" class="gallery-details">
<p class="gallery-details">
<strong class="gallery-details-title">query 1:</strong> 
<a class="gallery-details" 
    href="http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?CMD=search&amp;DB=pubmed&amp;term=$query1url" 
    onclick="return popup(this.href);">$query1enc
</a><br />
EOF

if ($query2) {
    print qq[<strong class="gallery-details-title">query 2:</strong> 
<a class="gallery-details"\n
    href="http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?CMD=search&amp;DB=pubmed&amp;term=$query2url"\n
    onclick="return popup(this.href);">$query2enc\n
</a><br />\n];}
}




sub viewDetails($ $ $ $ $ $ $) {

    my ($id, $so, $node, $edge, $prune, $node_list_ref, $edge_list_ref) = @_;

    my @node_list = @{$node_list_ref};
    my @edge_list = @{$edge_list_ref};

    print <<EOF;
<strong class="gallery-details-title">node type:</strong> $node<br />
<strong class="gallery-details-title">edge type:</strong> $edge<br />
EOF

if ($prune eq 'pruned') {
    print qq[<strong class="gallery-details-title">pruning option enabled</strong><br />\n];
}

    print <<EOF;
<br />
<strong class="gallery-details-title">
<a class="gallery-details" style="font-weight: bold;" href="node.pl?id=$id&amp;so=$so">text view</a>
</strong>
<br />
<strong class="gallery-details-title">nodes:</strong>
EOF

my $max = 5;

if (scalar(@node_list) < 5) { $max = scalar(@node_list); }

for (my $i = 0; $i < $max; $i++) { 
	print qq[<a class="gallery-details" href="node.pl?id=$id&amp;so=$so&amp;node=$node_list[$i]">$node_list[$i]</a> \n];
}

if ($max == 5) {
    print qq[<a class="gallery-details" style="font-weight: bold;" href="node.pl?id=$id&amp;so=$so">[...]</a><br />\n];
} 
else {
    print "<br />\n";
}

print qq[<strong class="gallery-details-title">edges:</strong>];

$max = 5;
if (scalar(@edge_list) < 5) {$max = scalar(@edge_list);}

for (my $i = 0; $i < $max; $i++) { 
    print qq[<a class="gallery-details" href="node.pl?id=$id&amp;so=$so&amp;edge=$edge_list[$i]">$edge_list[$i]</a> \n];
}

if ($max == 5) {
    print qq[<a class="gallery-details" style="font-weight: bold;" href="node.pl?id=$id&amp;so=$so#edgelist">[...]</a><br />\n];
}

print <<EOF;
</p>
</div>
EOF
}

sub footLinks($ $ $ $) {

    my ($id, $so, $topnet1, $topnet2) = @_;

    my $tYNA = "extNetworkUrl=$topnet1&amp;" . 
	"extNetworkName=$id&amp;" . 
	"extDirected=false&amp;" . 
	"extCategoryUrl=$topnet2";
    my $tYNAlink = TYNA . $tYNA;
print <<EOF;

<table class="foot">

<tr><th class="foot">Resources</th><th class="foot">Options</th></tr>

<tr>
<td class="foot">
<ul>
<li><a class="foot" href="http://www.adobe.com/svg/viewer/install/main.html">Adobe SVG Viewer</a></li>
<li><a class="foot" href="../firefox-svg.html">Enabling SVG in Firefox</a></li>
<li><a class="foot" href="http://www.svgi.org">Official W3C links</a></li>
<li><a class="foot" href="http://www.aisee.com/">aiSee Graph Software</a></li>
</ul>
</td>

<td class="foot"><a 
 href = "$tYNAlink"><img
 style="border:0px; align:right;" 
 src="../images/topnet.png" t
 title="Upload to tYNA" 
 alt="Upload to TopNet" /></a>
EOF

if (($so eq "o") || (! -e "../html/saved/$id.save")) {
    print <<EOF;
<br />
<a href="save.pl?id=$id"><img style="border:0px; align:right;" src="../images/exp-gallery.png" title="Save to Gallery" alt="Save to Gallery" /></a>

EOF
}

print <<EOF;
</td>
</tr>
</table>

EOF
}


sub getSource( $ $ ) {
    my ($so, $id) = @_;
    if (($so eq "r") || ($so eq "o")) { return (P_RECENT, "recent"); }
    elsif ($so eq "s") { return (P_SAVED, "saved"); }
    elsif (! $so) {
	if (-e P_SAVED . "/$id.save") { return (P_SAVED, "saved"); }
	elsif (-e P_RECENT . "/$id.txt") { return (P_RECENT, "recent"); }
	elsif (-e P_OUT . "/$id.txt") { return (P_OUT, "out"); }
	else {
	    print "Content-type: text/plain\n\n";
	    print "Query information not found.\n";
	    print  "If it was not explicitly saved, it may have expired.\n";
	    die "Expired query.";
	}
    }
}
