#!/usr/bin/perl

use IO::Socket::INET;
use DBI;
use XML::DOM;
use CGI (':param', ':Vars');
use Deaccent;
use SiteUtils;

use strict;


$| = 1;		# Autoflush

my %node_term; 	# node_term{node}@terms : hash of arrays to collect mesh terms for each pmid
my %term; 	# term{term}{node} = 1 or 0 : lookup table for checking if pmids share a term
my %term_freq;	# frequency of term occurrence
my %cite;	# "LastName Initials" for PubMed links in SVG graph
my %color;	# color{node_name} - color node blue (2), green (3), or yellow (4)

my ($id, $gdl, $pdf, $png, $png_small, $png_micro, $ps, $svg, $xml1, $xml2, $topnet1, $topnet2, $info, $report, $zip_file);
my ($h_id, $h_gdl, $h_pdf, $h_png, $h_png_small, $h_ps, $h_svg, $h_zip_file);

setFileNames();

my $query1 = param("query1");
my $query2 = param("query2");
my $node_type = param("node");
my $edge_type = param("edge");
my $zip_precision = param("zip");
my $prune = param("prune");
my $thick = param("thick");
my $so = "o";

# If an empty form has been submitted,
# reject.
if ( !$query1 ) {
    noQuery();
}

# Assume there's at least one query.

# Use a Captcha test to prevent spam.

my %p = Vars;
my $ok = ($query2 =~ /\*\*sesame\*\*$/);
if ($ok) { $query2 =~ s/\*\*sesame\*\*$//; }
else { $ok = captchaCheck(\%p); }
if (!$ok) { exit (0); }


# Assumed:  Query 1 has been submitted AND Captcha test was passed.

if ($node_type eq "pdb") {
    if ($query1) { $query1 = "PDB[si] " . $query1; }
    if ($query2) { $query2 = "PDB[si] " . $query2; }
}

if ($node_type eq "genbank") {
    if ($query1) { $query1 = "GENBANK[si] " . $query1; }
    if ($query2) { $query2 = "GENBANK[si] " . $query2; }
}

if ($node_type eq "swissprot") {
    if ($query1) { $query1 = "SWISSPROT[si] " . $query1; }
    if ($query2) { $query2 = "SWISSPROT[si] " . $query2; }
}

# Start processing query 1 and 
# providing output page.

pageHead("PubNet Query Results");
print <<EOF;
<div class="query-progress">
Fetching XML 1
EOF

    $query1 =~ s/[\n\r]/\ /g;
getXML($query1, $xml1, 900);
printCheck();


if ($query2) {
    print " 2 ";
    $query2 =~ s/[\n\r]/\ /g;
    getXML($query2, $xml2, 1000);
    printCheck();
} 

print "<br />\nParsing XML\n";

parseXML($xml1, 1);
printCheck();

if ($query2) {
    parseXML($xml2, 2);
    printCheck();
}

print "<br />\n";

my @node_names = sort(keys(%node_term));
my @term_names = sort(keys(%term));
my $nnodes = scalar(@node_names);
my $nterms = scalar(@term_names);

if ($nnodes > 500) {
    print <<EOF;
<span style="color: red;">Achtung!</span> 
Your query returned many results.  Please be patient.  
Or, see our <a style="color:#000099;" href="../doc.html#tips" onclick="return popup(this.href);">tips</a> 
for narrowing down your query.<br />
EOF
}

print qq[Compiling nodes: $nnodes ];

my @matrix;
my @row;
for (my $i = 0; $i < $nnodes; $i++) {push @row, 0 ;}
for (my $i = 0; $i < $nnodes; $i++) {push @matrix, [@row];}

my %node_index; # map incidence matrix index --> node name
                                        
for (my $i = 0; $i < $nnodes; $i++) {
    $node_index{$node_names[$i]} = $i;
}

open (GDL, ">$gdl");

printGDLHeader();

# iterate through each term, mark an edge between each node associated with that term
# in incidence matrix

my $edgecount = 0;

for (my $i = 0; $i < $nterms; $i++) {

    # get list of all terms for current node
    my @node_name_list = keys( %{$term{$term_names[$i]}} );

    # tally +1 for every pairwise combination of ids

    while (scalar(@node_name_list) > 1) {
	my $curr_node_name = shift @node_name_list;

	for (my $j = 0; $j < scalar(@node_name_list); $j++) {
	    $matrix[ $node_index{$curr_node_name} ][ $node_index{$node_name_list[$j]} ] += 1;
	    $matrix[ $node_index{$node_name_list[$j]} ][ $node_index{$curr_node_name} ] += 1;

	    if ($matrix[ $node_index{$curr_node_name} ][ $node_index{$node_name_list[$j]} ] == 1) {
		$edgecount++;
	    }
	}
    }
}

printCheck();

print qq[ edges: $edgecount ];

my $max_edge_weight = 0;

for (my $i = 0; $i < $nnodes; $i++) {
    for (my $j = 0; $j < $nnodes; $j++) {
	if ($max_edge_weight < $matrix[$i][$j]) {
	    $max_edge_weight = $matrix[$i][$j];
	}
    }
}

if ($max_edge_weight > 240) {$max_edge_weight = 220;}

my $inc;

if ($max_edge_weight == 0) {
    $inc = 120;
} 

else {
    $inc = 240 / $max_edge_weight;
}

my $num = 5;

for (my $i = 240; $i > 0; $i -= $inc) {
    my $j = sprintf "%d", $i;
    print GDL "colorentry $num: $j $j 240\n";
    $num++;
}

while ($num <= $max_edge_weight + 5) {
    print GDL "colorentry $num: 0 0 240\n";
    $num++;
}

open (TOP1, ">$topnet1");

my %exclude_list;

for (my $i = 0; $i < $nnodes; $i++) {
    for (my $j = $i; $j < $nnodes; $j++) {
	if ($matrix[$i][$j] > 0) {
	    my $thickness = $matrix[$i][$j];
	    my $color = $thickness + 5;
	    if ($color > $num) {$color = $num;}
	    if ($thick eq "on") {$thickness = 1;}
	    print GDL qq[edge:{source:"$i" target:"$j" thickness: $thickness color: $color } \n];
	    print TOP1 "$node_names[$i]\t$node_names[$j]\n";
	}
    }
}

close(TOP1);

for (my $i = 0; $i < $nnodes; $i++) {
    my $degree = 0;
    for (my $j = $0; $j < $nnodes; $j++) {
	if ($matrix[$i][$j] > 0) {
	    $degree++;
	}
    }
    if (!$degree) {$exclude_list{$i} = 1;}
}

my $omit = scalar(keys(%exclude_list));

for (my $i = 0; $i < $nnodes; $i++) {
    my $node_name = $node_names[$i];
    my $color = $color{$node_name};
    my $url = "../cgi-bin/node.pl?id=$id&node=$node_name";
    if ($prune eq "on" && $exclude_list{$i}) {
	next;
    } 
    else {
	print GDL qq[node:{title:"$i" info1:"$node_name" label:"$node_name" info3:"href:$url;" color:$color}\n];
    }
}

print GDL "}\n";
close(GDL);

my (@blue, @yellow, @green);
open (TOP2, ">$topnet2");
for (my $i = 0; $i < $nnodes; $i++) {
	if ($color{$node_names[$i]} == 2) {push @blue, $node_names[$i];}
	if ($color{$node_names[$i]} == 3) {push @green, $node_names[$i];}
	if ($color{$node_names[$i]} == 4) {push @yellow, $node_names[$i];}
}

if (scalar(@blue)) {print TOP2 "blue\t" . (join "\t", @blue) . "\n";}
if (scalar(@yellow)) {print TOP2 "yellow\t" . (join "\t", @yellow) . "\n";}
if (scalar(@green)) {print TOP2 "green\t" . (join "\t", @green) . "\n";}

close(TOP2);

printCheck();
print "<br />\n";

if ($prune eq "on") {
    print qq[Omitting $omit unconnected nodes.<br />];
}

print qq[Generating output: svg ];
system("/usr/local/bin/aisee -color -silent -svgoutput $svg $gdl");
printCheck();

print "ps";
system("/usr/local/bin/aisee -color -silent -psoutput $ps $gdl");
printCheck();

print "pdf";
my $cmd = qq[perl -i -p -e 's/Created with aiSee V2.0 \\(ERP-Version\\) \\(c\\) 2000 AbsInt Angewandte Informatik GmbH. Commercial use prohibited!/Created with PubNet./ig' $ps];
system($cmd);
system("ps2pdf $ps $pdf");
system("convert -scale 400x400 $ps $png_small");
system("convert -scale 100x100 $ps $png_micro");
printCheck();

print "png";
$cmd = "convert $ps $png";
system($cmd);
printCheck();

print qq[\n</div>\n\n];

# sanitize svg
if (-e $svg) {
	# Remove aiSee link from SVG 
	$cmd = qq[perl -i -p -e 's/<svg:g id=\\"icsticker\\">/<!--<svg:g id=\\"icsticker\\">/ig' $svg];
	system($cmd);	
	$cmd = qq[perl -i -p -e 's/<svg:animate xlink:href=\\"\\#icbackgr\\".*remove\\"\\//--/ig' $svg];
	system($cmd);

	# Add spaces to SVG so names are centered
	$cmd = qq[perl -i -p -e 's/1999\\/xlink\\">/1999\\/xlink\\" xml:space=\\"preserve\\">/ig' $svg];
	system($cmd);
	$cmd = qq[perl -i -p -e 's/<svg:text (.*) font-family=\\"Helvetica\\" font-size=\\"14\\" fill=\\"\\#000\\">(.*)<\\/svg:text>/<svg:text \$1 font-family=\\"Helvetica\\" font-size=\\"14\\" fill=\\"\\#000\\">  \$2<\\/svg:text>/ig' $svg];
	system($cmd);

}

# Record data structure for node browsing
open (REPORT, ">$report");

foreach my $key (keys(%node_term)) {
	my $terms = join("\t", @{$node_term{$key}});
	print REPORT "$key\t$terms\t$color{$key}\n";
}
close(REPORT);

# Record request parameters and brief meta description
open (INFO, ">$info");
print INFO "$query1\t$query2\n$node_type\t$edge_type\n";
print INFO "nodes:\t$nnodes\nedges:\t$edgecount\n";

if ($prune eq "on") {
  print INFO "pruned\n\n";
} 
else {
  print INFO "\n";
}

close(INFO);

system("zip -j -q " . P_OUT . "/$id " . P_OUT . "/$id*.*");

# Add to recent gallery

open(LAST, "<" . LAST16);
my @last16 = <LAST>;
close(LAST);

if (scalar(@last16) >= 16) {
    my $too_old = shift @last16;
    chomp $too_old;
    unlink glob P_RECENT . "/$too_old*.*";
}
push @last16, "$id\n";
open(LAST, ">" . LAST16);
print LAST @last16;
close LAST;
system("cp " . P_OUT . "/$id*.* " . P_RECENT);



# Get rid of files more than a day old.
system("find . -wholename '" . P_OUT . "/*' -mtime +1 -delete");


my @node_list = sort keys(%node_term);
my @edge_list = sort keys(%term);


# Present output on page.

viewQuery($id, $h_id, $query1, $query2, $node_type, $edge_type);

viewDetails($id, $so, $node_type, $edge_type, $prune, \@node_list, \@edge_list);

footLinks($id, $so, $topnet1, $topnet2);

pageFoot();

# End of main.



sub hashDescSortNum {$term_freq{$b} <=> $term_freq{$a};}

sub ProperCase {join(' ',map{ucfirst(lc("$_"))}split(/\s/,$_[0]));}

sub setFileNames {
	$id = `date +%y%m%d%H%M%S`;
	$id =~ s/\s//;
	my $p_prefix = P_OUT . "/$id";
	my $h_prefix = H_OUT . "/$id";
	$h_id = $h_prefix;
	$gdl = $p_prefix . ".gdl";
	$h_gdl = $h_prefix . ".gdl";
	$pdf = $p_prefix . ".pdf";
	$h_pdf = $h_prefix . ".pdf";
	$png = $p_prefix . ".png";
	$h_png = $h_prefix . ".png";
	$png_small = $p_prefix . "_small.png";
	$h_png_small = $h_prefix . "_small.png";
	$png_micro = $p_prefix . "_micro.png";
	$ps  = $p_prefix . ".ps";
	$h_ps = $h_prefix . ".ps";
	$svg = $p_prefix . ".svg";
	$h_svg = $h_prefix . ".svg";
	$xml1 = $p_prefix . "-1.xml";
	$xml2 = $p_prefix . "-2.xml";
	$topnet1 = $p_prefix . ".int";
	$topnet2 = $p_prefix . ".cat";
	$info = $p_prefix . ".txt";
	$report = $p_prefix . ".report";
	$zip_file = $p_prefix . ".zip";
	$h_zip_file = $h_prefix . ".zip";
}


sub getXML( $ $ $ ) {
    my ($query, $xml, $dispmax) = @_;

    my $NCBI = IO::Socket::INET->new (PeerPort => '80', Proto => 'tcp', PeerAddr => 'www.ncbi.nlm.nih.gov') 
	or die "Aw crap, can't connect to NCBI!\n";

    $NCBI->autoflush();

    my $queryenc = URLEncode($query);
    my $qstring = "GET /sites/entrez?Db=pubmed&Cmd=PureSearch&Term=" . $queryenc . "&dispmax=$dispmax&relpubdate=No+Limit\n";

    print $NCBI $qstring;

    my @papers = ();
    while (<$NCBI>) {
	if (/PMID:\ /) {
	    chomp;
	    s/.*PMID:\ //;
	    s/\ \[.*//;
	    push (@papers, $_);
	}
    }

    close($NCBI);

    open(XML, "> $xml");
    my $list = join(",", @papers);
    my $pubmed = `wget -q -O - \"http://www.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=PubMed&retmode=xml&id=$list\"`;
    $pubmed =~ s/<FirstName>/<ForeName>/g;
    $pubmed =~ s/<\/FirstName>/<\/ForeName>/g;
    print XML $pubmed;
    close(XML);
}


sub parseXML($ $) {

    my ($input, $pass) = @_;

    my $parser = new XML::DOM::Parser;

    my $doc = $parser->parsefile($input);

    my $nodes = $doc->getElementsByTagName("MedlineCitation");
    my $n = $nodes->getLength;

    if (!$n) {
	print qq[<img style="border-width:0;" src="../images/x.png" title="Failure" alt="Failed" /><br />];
	print <<EOF;

<p>Query $pass returned zero results.  You may want to practice on <a class="main" href="http://www.ncbi.nlm.nih.gov/entrez/query.fcgi" 
    onclick="return popup(this.href);">PubMed</a> first.</p>
<hr />
<p><big>NOTE - around Thu Oct 20 22:10 EDT 2005, PubMed started returning an error in response 
to XML requests (and all other requests requiring the PubMed article server). Fortuitously, I actually watched this happen and was 
able to verify that PubNet isn't broken. I've emailed NCBI about the problem.<br /><br />If you see this message and you are able to 
retrieve XML results from the NCBI page (i.e., your query gives results, AND you can switch to XML view using the drop-down menu), 
then there may be connection issues with PubNet.  However, if PubMed is still broken, then so is PubNet.  Sorry for the 
inconvenience.</big><br />--smd 22:50, 20 October 2005 (EDT)</p>

EOF
pageFoot();
	exit;
    }
    for (my $i = 0; $i < $n; $i++) {
	my @node_name_list;
	my @termlist;
	my $curr_node = $nodes->item($i);

	# Skip papers with no authors by default
	if (! $curr_node->getElementsByTagName("AuthorList")->getLength) {next;}

	# Extract PMID
	if ($node_type eq "paper" || ($node_type eq "author"  && $edge_type eq "coauthor")) {
	    my $PMID_node = ($curr_node->getElementsByTagName("PMID"))[0];
	    my $PMID = $PMID_node->getFirstChild->getData();
	    
	    if ($node_type eq "paper") {
		$cite{$PMID} = $PMID;
		push @node_name_list, $PMID;	# Node: paper
	    }
	    else {
		push @termlist, $PMID;		# Edge: co-author
	    }
	}

	# Extract MeSH Terms
	if ($edge_type eq "mesh") {
	    my $MeshHeadingList = $curr_node->getElementsByTagName("MeshHeadingList");
	    my $j = 0;

	    if ($MeshHeadingList->getLength) {
		my $MeshHeadingList_node = $MeshHeadingList->item(0);
		my $MeshHeadings = $MeshHeadingList_node->getElementsByTagName("MeshHeading");
		$j = $MeshHeadings->getLength;

		for (my $i = 0; $i < $j; $i++) {
		    my $MeshHeading_node = $MeshHeadings->item($i);
		    my $Descriptor_node = ($MeshHeading_node->getElementsByTagName("DescriptorName"))[0];
		    my $Qualifier_list = $MeshHeading_node->getElementsByTagName("QualifierName");
		    
		    my $DescriptorName;
		    my $MajorTopicYN;

		    if ($Descriptor_node) {
			if ($Descriptor_node->hasChildNodes()) {
			    $DescriptorName = $Descriptor_node->getFirstChild->getData();
			    $MajorTopicYN = $Descriptor_node->getAttribute("MajorTopicYN");
			} 
			else {
			    next;
			}

			my $nqual = $Qualifier_list->getLength;

			for (my $k = 0; $k < $nqual; $k++) {
			    my $Qualifier_node = $Qualifier_list->item($k);
			    my $QualifierName = $Qualifier_node->getFirstChild->getData();
			    my $MajorQualYN = $Qualifier_node->getAttribute("MajorTopicYN");
			    if ($MajorQualYN eq "Y") {
				my $TermName = "$DescriptorName - $QualifierName";
				push @termlist, $TermName;
			    }
			}
		    } 
		    else {
			next;
		    }
		    if ($MajorTopicYN eq "Y") {
			push @termlist, $DescriptorName;
		    }
		}
	    }	
	}


	# Extract Authors if node = author or edge = coauthorship
	if (($node_type eq "author") || (($node_type ne "author") && ($edge_type eq "coauthor"))) {

	    my $AuthorList = ($curr_node->getElementsByTagName("AuthorList"))[0];
	    my $AuthorNodes = $AuthorList->getElementsByTagName("Author");

	    my $j = $AuthorNodes->getLength;

	    my @namelist;

	    for (my $i = 0; $i < $j; $i++) {
		my $Author = $AuthorNodes->item($i);

		my $LastName_node = ($Author->getElementsByTagName("LastName"))[0];
		my $LastName;

		if ($LastName_node) {
		    if ($LastName_node->hasChildNodes()) {
			$LastName = $LastName_node->getFirstChild->getData();
		    } 
		    else {
			next;
		    }
		}

		my $Initial_node = ($Author->getElementsByTagName("Initials"))[0];
		my $Initial;

		if ($Initial_node) {
		    if ($Initial_node->hasChildNodes()) {
			$Initial = $Initial_node->getFirstChild->getData();
		    } 
		    else {
			next;
		    }
		}

		my $char = substr($Initial, 0, 1);
		$char = DeAccent($char);

		$LastName = ProperCase($LastName);
		$LastName = DeAccent($LastName);

		my $name = "$char $LastName";

		$cite{$name} = "$LastName%20$Initial";

		if ($node_type eq "author") {
		    push @node_name_list, $name;
		} 
		else {
		    push @termlist, $name;
		}
	    }
	}

	# Extract PDB or GENBANK ids
	if ($node_type eq "pdb" || $node_type eq "genbank" || $node_type eq "swissprot") {
	    my $DataBankList = $curr_node->getElementsByTagName("DataBank");
	    my $j = 0;

	    for (my $i = 0; $i < $DataBankList->getLength; $i++) {

		my $DataBankList_node = $DataBankList->item($i);
		my $DataBankName_node = ($DataBankList_node->getElementsByTagName("DataBankName"))[0];
		my $DataBankName = $DataBankName_node->getFirstChild->getData();

		if (($node_type eq "pdb" && $DataBankName ne "PDB") || 
		    ($node_type eq "genbank" && $DataBankName ne "GENBANK") ||
		    ($node_type eq "swissprot" && $DataBankName ne "SWISSPROT")) {
		    next;
		}

		my $AccessionNumberList = $DataBankList_node->getElementsByTagName("AccessionNumber");

		for (my $j = 0; $j < $AccessionNumberList->getLength; $j++) {
		    my $AccessionNumberList_node = $AccessionNumberList->item($j);
		    my $AccessionNumber = $AccessionNumberList_node->getFirstChild->getData();
		    push @node_name_list, $AccessionNumber;
		    $cite{$AccessionNumber} = $AccessionNumber;
		}
	    }	
	}

	# Extract Zip codes
	if ($edge_type eq "zip") {
	    my $zip;

	    my $AffList = $curr_node->getElementsByTagName("Affiliation");

	    if ($AffList->getLength) {
		my $Aff_node = ($curr_node->getElementsByTagName("Affiliation"))[0];
		my $Aff = $Aff_node->getFirstChild->getData();

		if ($zip_precision eq "3") {
		    $Aff =~ /\W(\d{3})\d{2}\W/;
		    $zip = $1;
		} 
		elsif ($zip_precision eq "4") {
		    $Aff =~ /\W(\d{4})\d{1}\W/;
		    $zip = $1;
		} 
		else {
		    $Aff =~ /\W(\d{5})\W/;
		    $zip = $1;
		}
		if ($zip) {push @termlist, $zip;} 
	    }
	}

	# Pair up nodes with edges
	for (my $i = 0; $i < scalar(@node_name_list); $i++) {
	    push @{$node_term{$node_name_list[$i]}}, @termlist;

	    for (my $j; $j < scalar(@termlist); $j++) {
		$term{$termlist[$j]}{$node_name_list[$i]} = 1;
		$term_freq{$termlist[$j]} += 1;
	    }

	    # Mark color
	    if ($pass == 1) {
		$color{$node_name_list[$i]} = 2;
	    } 
	    elsif ($pass == 2) {
		if ($color{$node_name_list[$i]} == 2 || $color{$node_name_list[$i]} == 3) {
		    $color{$node_name_list[$i]} = 3;
		} 
		else {
		    $color{$node_name_list[$i]} = 4;
		}
	    }
	}

    } # matches "for (my $i = 0; $i < $n; $i++)"
}

sub printGDLHeader {
print GDL <<EOF;
graph: {title:"Publication Network Graph"
color          : white
scaling        : maxspect
layoutalgorithm: forcedir
gravity        : 1.3
repulsion      : 200
attraction     : 120
magnetic_field1: polar
magnetic_field2: polar
crossing_optimization: yes
crossing_phase2: yes
display_edge_labels: yes
xbase: 50
ybase: 50
colorentry 1: 0 0 0
colorentry 2: 128 128 255
colorentry 3: 128 255 128
colorentry 4: 255 255 128
edge.color:1
edge.arrowstyle:none
edge.thickness:1
node.fontname:"helvR12"
node.borderwidth:0
node.textcolor:1
node.color:1
node.shape:ellipse
node.height:24
node.width:68
node.importance:2
EOF

}

sub printCheck {
    print qq[<img style="border-width:0;" src="../images/check.png" title="Success" alt="Succeeded" /> ];
}


sub noQuery {
    pageHead("PubNet - Error: Missing Query");
    print <<EOF;
<h1>Missing Query</h1>
<p>Please enter at least Query 1.<br />
See <a style="color:#8080ff;" href="../doc.html#tips">documentation</a> for tips.</p>
EOF
pageFoot();
    die "Query 1 omitted.\n";
}
