#!/usr/bin/perl

use CGI ':param';
use SiteUtils;
use strict;


my $id = param("id");
my $node = param("node");
my $edge = param("edge");
my $esort = param("esort");
my $nsort = param("nsort");
my $so = param("so");

if (!$id) {
        print "Content-type: text/plain\n\n";
        print "Must supply id and node.\n";
        die "Must supply id.";
}

my ($source, $sourcename) = getSource($so, $id);

if (!$esort) {$esort = 0;}
if (!$nsort) {$nsort = 0;}

my %nodes;
my %col;
my %col_map = (2 => "bdot", 16 => "bdot", 4 => "ydot", 19 => "ydot", 3 => "gdot", 18 => "gdot");

my %term_node;
my %term_freq;

my $txt = "$source/$id.txt";
open(TXT, "<$txt");
<TXT>;
my $types = <TXT>;
my ($nodetype, $edgetype) = split /\t/, $types;

my $report = "$source/$id.report";
open(REPORT, "<$report");

while (<REPORT>) {
	my @tmp = split /\t/;
	my $node = shift @tmp;
	my $color = pop @tmp;
	chomp $color;

	# green nodes get edge terms counted twice
	# here's a quick hack to avoid recounting them
	my %uniq;
	for (my $i = 0; $i < scalar(@tmp); $i++) {
		$uniq{$tmp[$i]} = 1;
	}
	@tmp = keys(%uniq);

	my $list = join "\t", @tmp; 
	$nodes{$node} = $list;
	$col{$node} = $color;
}
close(REPORT);

my %degree;

# collect node->edge mappings
foreach my $key (keys(%nodes)) {
	my $list = $nodes{$key};
	my @terms = split /\t/, $list;
	for (my $i = 0; $i < scalar(@terms); $i++) {
		$term_node{$terms[$i]}{$key} = 1;
		$term_freq{$terms[$i]} += 1;
	}
	$degree{$key} = scalar(@terms);
}

# extract neighbors for this particular node
my %neighbor;
$neighbor{$node} = 0;
my $num = 1;
my $list = $nodes{$node};
my @terms = split /\t/, $list;
for (my $i = 0; $i < scalar(@terms); $i++) {
	foreach my $key (keys(%{$term_node{$terms[$i]}})) {
		if ($key ne "$node") {
			$neighbor{$key} = $num++;
		}
	}
}


pageHead("PubNet - Node View");

if (!$node && !$edge) {

    print <<EOF;

<h1 class="node">Text Summary of Nodes</h1>
<p class="node">Click for a detailed view of neighbors and outgoing edges.</p>

<p class="node sort">sort by 
<a class="node sort" href="node.pl?id=$id&amp;so=$so&amp;esort=$esort">name</a> | 
<a class="node sort" href="node.pl?id=$id&amp;so=$so&amp;nsort=1&amp;esort=$esort">degree</a>
</p>

<p class="nodelist">
EOF

	my @all;
	my @all_nodes;
	if ($nsort) {
		@all = sort {$degree{$b} <=> $degree{$a}} keys(%nodes);
	} else {
		@all = sort keys(%nodes);
	}


	my $alt = 1;
	foreach my $key (@all) {
		my $dot = $col_map{$col{$key}};
		if ($nsort) {
			print qq[<img src="../images/$dot.png" title="$dot" alt="$dot" /> 
                           <a class="node grey" href="node.pl?id=$id&amp;so=$so&amp;node=$key">$key ($degree{$key})</a><br />\n];
		} else {
			print qq[<img src="../images/$dot.png" title="$dot" alt="$dot" /> 
                          <a class="node grey" href="node.pl?id=$id&amp;so=$so&amp;node=$key">$key</a> |\n];
		}
	}

    print <<EOF;
</p>
<hr class="node" />
<h1 class="node" id="edgelist">Text Summary of Edges</h1>
<p class="node">Click for a list of nodes sharing this edge type.</p>
<p class="node sort">sort by 
<a class="node sort" href="node.pl?id=$id&amp;so=$so&amp;nsort=$nsort">name</a> | 
<a class="node sort" href="node.pl?id=$id&amp;so=$so&amp;nsort=$nsort&amp;esort=1#edgelist">frequency</a>
</p>

<p class="nodelist">
EOF
	my @all_terms;
	if ($esort) {
		@all_terms = sort {$term_freq{$b} <=> $term_freq{$a}} keys(%term_node);
	} else {
		@all_terms = sort keys(%term_node);
	}

	$alt = 1;
	foreach my $key (@all_terms) {
		my $class;
		if ($alt % 2) { $class = "dark"; } else { $class = "light"; }
		$alt++;
		if ($esort) {
			print qq[<a class="node $class" href="node.pl?id=$id&amp;so=$so&amp;edge=$key">$key ($term_freq{$key})</a><br />\n];
		} else {
			print qq[<a class="node $class" href="node.pl?id=$id&amp;so=$so&amp;edge=$key">$key</a> \n];
		}
	}
    print qq[</p>];

} elsif ($node) {
	my $ndot = $col_map{$col{$node}};
	my @nlist = sort keys(%neighbor);
	my $url = "http://www.ncbi.nlm.nih.gov/gquery/gquery.fcgi?term=$node";
	my $pmurl = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?CMD=search&amp;DB=pubmed&amp;term=$node";

	print <<EOF;
<h1 class="node">$node</h1>
<div class="dblinks">
EOF

	if ($nodetype eq "pdb") {
	  my $pdburl = "http://www.rcsb.org/pdb/cgi/explore.cgi?pdbId=$node";
	  print qq[<a href="$pdburl" onclick="return popup(this.href);"><img style="vertical-align: bottom; border:0px;" src="../images/pdb.png" title="PDB" alt="PDB" /></a> &nbsp;];
	}

	if ($nodetype eq "swissprot") {
	  my $swissurl = "http://us.expasy.org/uniprot/$node";
	  print qq[<a href="$swissurl" onclick="return popup(this.href);"><img style="vertical-align: bottom; border:0px;" src="../images/uniprot.png"
                     title="SwissProt" alt="SwissProt" /></a> &nbsp;];
	}

	print <<EOF;
<a href="$pmurl" onclick="return popup(this.href);"><img style="vertical-align: bottom; border:0px;" src="../images/pm.png" title="PubMed" alt="PubMed" /></a> &nbsp;
<a href="$url" onclick="return popup(this.href);"><img style="vertical-align: bottom; border:0px;" src="../images/entrez.png" title="PubMed Entrez" alt="PubMed Entrez" /></a>
</div>

<p class="node sort">[this node : edges]</p>
<p class="nodelist">
<img src="../images/$ndot.png" title="$ndot" alt="$ndot" />$node :
EOF

	my @list = $nodes{$node};
	my @terms = split /\t/, $list;
	for (my $i = 0; $i < scalar(@terms); $i++) {
		print qq[<a class="node red" href="node.pl?id=$id&amp;so=$so&amp;edge=$terms[$i]">$terms[$i]</a>];
		if ($i < (scalar(@terms) - 1)) {
		print ", ";
		}
	}

	print <<EOF;
</p>
<p class="node sort">[adjacent node : edges]</p>
<p class="nodelist">
EOF

	foreach my $key (@nlist) {
		if ($key eq $node) {next};
		my $dot = $col_map{$col{$key}};
		print qq[<img src="../images/$dot.png" title="$dot" alt="$dot" /> <a class="node grey" href="node.pl?id=$id&amp;so=$so&amp;node=$key">$key</a> : \n];

		my $list = $nodes{$key};
		my @terms = split /\t/, $list;
		for (my $i = 0; $i < scalar(@terms); $i++) {

			if ($term_node{$terms[$i]}{$node} == 1) {
				print qq[<a class="node red" href="node.pl?id=$id&amp;so=$so&amp;edge=$terms[$i]">$terms[$i]</a>];
			} else {
				print qq[<a class="node lightgrey" href="node.pl?id=$id&amp;so=$so&amp;edge=$terms[$i]">$terms[$i]</a>];
			}


			if ($i < (scalar(@terms) - 1)) {
			print ", ";
			}
		}
		print qq[<br />\n];
	}
	print <<EOF; 

</p>
<p class="node sort">
[common edges between this node (<img src="../images/$col_map{$col{$node}}.png" title="Node" alt="Node" /> $node) 
     and adjacent nodes <span style="color: red;">shown in red</span>]</p>
EOF
	
} 
else {
    my $url = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?CMD=search&amp;DB=pubmed&amp;term=$edge";
print <<EOF;
<h1 class="node">$edge</h1>
<div class="dblinks"><a href="$url" onclick="return popup(this.href);"><img style="border:0px;" 
 src="../images/pubmed.png" title="PubMed" alt="PubMed" /></a></div>
<p class="nodelist">
EOF
    my $alt = 0;
foreach my $key (sort keys(%{$term_node{$edge}})) {
    my $dot = $col_map{$col{$key}};
    print qq[<img src="../images/$dot.png" title="$dot" alt="$dot" /> 
	     <a class="node grey" href="node.pl?id=$id&amp;so=$so&amp;node=$key">$key</a> |\n];
	}

print qq[</p>\n];
}

print qq[<hr class="node"/>\n];

pageFoot();





